Attribute VB_Name = "modConfig"
Option Explicit
Option Compare Binary
Option Base 0


' Path to configuration file
Public ConfigPath As String


' Load and add a single engine configuration file
Public Sub AddEngine(ByVal ConfigFile As String)

    Dim A As Long
    Dim MyConfig As New clsConfiguration
    Dim TempRef As Dictionary
    
    
    MyConfig.LoadConfiguration ConfigFile
    
    ' Validate
    If MyConfig.Root.Item("type") <> "LaunchTool engine file" Then Exit Sub
    
    ' Add new engine
    ReDim Preserve Engines(nEngines)
    nEngines = nEngines + 1
        
    ' Copy
    With Engines(nEngines - 1)
        
        ' Basic info
        .Executable = MyConfig.Root.Item("executable")
        .Name = MyConfig.Root.Item("realname")
        .DemoExt = MyConfig.Root.Item("demoext")
        
        
        ' Get parameters, dimension
        Set TempRef = MyConfig.Root(True).Item("parameters")
        ReDim .Parameters(TempRef.Count - 1) As ParameterType
        
        ' Read each parameter
        For A = 0 To TempRef.Count - 1
            .Parameters(A).Description = TempRef.Items(A)
            .Parameters(A).Parameter = "-" & TempRef.Keys(A)
        Next A
        
        
        ' Get supported filetypes, dimension
        Set TempRef = MyConfig.Root(True).Item("filetypes")
        ReDim .FileTypes(TempRef.Count - 1) As FileType
        
        ' Read each parameter
        For A = 0 To TempRef.Count - 1
            .FileTypes(A).Description = TempRef.Items(A)
            .FileTypes(A).Extension = TempRef.Keys(A)
        Next A
        
    End With

End Sub


' Load all available engine configuration files form a directory
Public Sub LoadEngines(ByVal ConfigDir As String)

    Dim FileName As String
    
    
    ' Retrieve the first entry
    FileName = Dir$(ConfigDir & "\*.cfg")
    
    ' No engine configs?
    If FileName = "" Then
        MsgBox "No engine configuration files were found. Please check if you installed LaunchTool properly.", vbOKOnly Or vbCritical, MsgBoxTitle
        modMain.Quit
        Exit Sub
    End If
    
    
    Do
        
        ' Load and add the configuration
        AddEngine ConfigDir & "\" & FileName
        
        ' Retrieve the next entry
        FileName = Dir$()
        
    Loop While FileName <> ""

End Sub


' Initialise new configuration
Public Sub InitConfig()

    Set Config = New clsConfiguration
    ConfigPath = App.Path & "\launchtool.cfg"
    
    
    Config.Root.Add "type", "LaunchTool Configuration file"
    
    Config.Root.Add "engine", -1
    Config.Root.Add "iwad", -1
    Config.Root.Add "skill", 2
    Config.Root.Add "map", -1
    
    Config.Root.Add "fastmonsters", 0
    Config.Root.Add "nomonsters", 0
    Config.Root.Add "respawnmonsters", 0
    Config.Root.Add "deathmatch", 0
    Config.Root.Add "parameters", ""
    
    Config.Root.Add "lastdir", App.Path
    
    Config.Root.Add "iwads", New Dictionary
    Config.Root.Add "engines", New Dictionary
    Config.Root.Add "pwads", New Dictionary
    
End Sub


' Load the program's configuration
Public Sub LoadConfig()

    Dim A As Long
    Dim PipeOffset As Long
    Dim TempDic As Dictionary
    Dim TempEng As Dictionary
    
    
    ' Load
    Config.LoadConfiguration ConfigPath
    
    ' Validate
    If Config.Root.Item("type") <> "LaunchTool Configuration file" Then
        MsgBox "The configuration file was corrupted.", vbOKOnly Or vbCritical, MsgBoxTitle
        modMain.Quit
        Exit Sub
    End If
    
    
    ' Get IWADs
    Set TempDic = Config.Root.Item("iwads")
    
    ' Redimension
    ReDim IWADs(TempDic.Count)
    nIWADs = TempDic.Count
    
    ' Read
    If nIWADs > 0 Then
        For A = 0 To nIWADs - 1
            PipeOffset = InStr(1, TempDic.Items(A), "|")
            IWADs(A).Name = Left$(TempDic.Items(A), PipeOffset - 1)
            IWADs(A).FileName = Mid$(TempDic.Items(A), PipeOffset + 1)
        Next A
    End If
    
    
    ' Get engines
    Set TempDic = Config.Root.Item("engines")
    
    ' Read data
    For A = 0 To nEngines - 1
        If TempDic.Exists(modUtil.SpaceConv(Engines(A).Name)) Then
            Set TempEng = TempDic.Item(modUtil.SpaceConv(Engines(A).Name))
            Engines(A).ExecutablePath = TempEng.Item("path")
            modUtil.ParseOptions TempEng.Item("options"), Engines(A).Options
        End If
    Next A
    
    
    ' Load filelist
    Set TempDic = Config.Root.Item("pwads")
    FileList.Clear
    
    ' Read data
    For A = 0 To TempDic.Count - 1
        FileList.Add TempDic.Items(A)
    Next A

End Sub


' Save the program's configuration
Public Sub SaveConfig()

    Dim A As Long
    Dim TempDic As Dictionary
    Dim TempEng As Dictionary
    
    
    ' Copy IWADs
    Set TempDic = Config.Root.Item("iwads")
    TempDic.RemoveAll
    
    For A = 0 To nIWADs - 1
        TempDic.Add Trim$(Str$(A)), IWADs(A).Name & "|" & IWADs(A).FileName
    Next A
    
    
    ' Save engines
    Set TempDic = Config.Root.Item("engines")
    TempDic.RemoveAll
    
    For A = 0 To nEngines - 1
        Set TempEng = New Dictionary
        
        ' Add settings
        TempEng.Add "path", Engines(A).ExecutablePath
        TempEng.Add "options", modUtil.OptionsString(Engines(A).Options)
        
        TempDic.Add modUtil.SpaceConv(Engines(A).Name), TempEng
    Next A
    
    
    ' Save filelist
    Set TempDic = Config.Root.Item("pwads")
    TempDic.RemoveAll
    
    For A = 0 To FileList.Count
        TempDic.Add Trim$(Str$(A)), FileList.Item(A)
    Next A
    
    
    ' Save the config
    Config.SaveConfiguration ConfigPath, vbCrLf, True

End Sub
