Attribute VB_Name = "modDialog"
Option Explicit
Option Compare Binary
Option Base 0


' File Open/Save structure
Private Type cdlOpenFileName
    lStructSize As Long
    Owner As Long
    hInstance As Long
    Filter As String
    CustomFilter As String
    MaxCustFilter As Long
    FilterIndex As Long
    File As String
    MaxFile As Long
    FileTitle As String
    MaxFileTitle As Long
    InitialDir As String
    Title As String
    flags As Long
    FileOffset As Integer
    FileExtension As Integer
    DefExt As String
    CustData As Long
    Hook As Long
    TemplateName As String
End Type


' Folder structure
Private Type BrowseInfo
    hwndOwner As Long
    pIDLRoot As Long
    pszDisplayName As Long
    lpszTitle As Long
    ulFlags As Long
    lpfnCallback As Long
    lParam As Long
    iImage As Long
End Type


' Constants
Public Enum cdlFileOpenConstants
    cdlOFNAllowMultiselect = 512
    cdlOFNCreatePrompt = 8192
    cdlOFNExplorer = 524288
    cdlOFNExtensionDifferent = 1024
    cdlOFNFileMustExist = 4096
    cdlOFNHelpButton = 16
    cdlOFNHideReadOnly = 4
    cdlOFNLongNames = 2097152
    cdlOFNNoChangeDir = 8
    cdlOFNNoDereferenceLinks = 1048576
    cdlOFNNoLongNames = 262144
    cdlOFNNoReadOnlyReturn = 32768
    cdlOFNNoValidate = 256
    cdlOFNOverwritePrompt = 2
    cdlOFNPathMustExist = 2048
    cdlOFNReadOnly = 1
    cdlOFNShareAware = 16384
End Enum


' Win32 API calls
Private Declare Function GetOpenFileName Lib "comdlg32.dll" Alias "GetOpenFileNameA" (pOpenfilename As cdlOpenFileName) As Long
Private Declare Function GetSaveFileName Lib "comdlg32.dll" Alias "GetSaveFileNameA" (pSavefilename As cdlOpenFileName) As Long
Private Declare Function SHBrowseForFolder Lib "shell32.dll" (lpbi As BrowseInfo) As Long
Private Declare Function SHGetPathFromIDList Lib "shell32.dll" (ByVal pidList As Long, ByVal lpBuffer As String) As Long
Private Declare Function lstrcat Lib "kernel32.dll" Alias "lstrcatA" (ByVal lpString1 As String, ByVal lpString2 As String) As Long
Private Declare Sub CoTaskMemFree Lib "ole32.dll" (ByVal hMem As Long)


' Select a folder
Public Function SelectFolder(hWnd As Long, Prompt As String) As String
     
    Dim intNull As Integer
    Dim lngIDList As Long
    Dim lngResult As Long
    Dim strPath As String
    Dim udtBI As BrowseInfo
    
    
    ' Set browse information
    With udtBI
        .hwndOwner = hWnd
        .lpszTitle = lstrcat(Prompt, "")
        .ulFlags = 1
    End With
    
    ' Select folder
    lngIDList = SHBrowseForFolder(udtBI)
    
    ' Check if not cancelled
    If lngIDList Then
    
        ' Get the selected path
        strPath = String$(260, 0)
        lngResult = SHGetPathFromIDList(lngIDList, strPath)
        
        ' Free used memory
        Call CoTaskMemFree(lngIDList)
        
        ' Remove null terminator
        strPath = StripNullChar(strPath)
    
    End If
    
    ' Return the result
    SelectFolder = strPath
     
End Function


' Open (or select) a file
Public Function OpenFile(ByVal hWnd As Long, ByRef DialogTitle As String, ByRef Filter As String, ByRef InitFilename As String, ByVal flags As cdlFileOpenConstants) As String
     
    Dim FileStruct As cdlOpenFileName
    Dim intNull As Integer
    
    
    With FileStruct
    
        ' Set all dialog parameters
        .lStructSize = Len(FileStruct)
        .hInstance = App.hInstance
        .Owner = hWnd
        .Title = DialogTitle
        .Filter = Replace$(Filter, "|", vbNullChar)
        .flags = flags
        .File = String$(257, Chr$(0)) & vbNullChar
        .MaxFile = Len(.File)
        .InitialDir = Config.Root.Item("lastdir")
        If InitFilename <> "" And Dir(InitFilename) <> "" Then .InitialDir = Left$(InitFilename, Len(InitFilename) - Len(Dir(InitFilename)))
        
        ' Show the dialog and return the result
        If GetOpenFileName(FileStruct) Then
            OpenFile = StripNullChar(.File)
            
            ' Remember the last used dir
            If (flags And cdlOFNAllowMultiselect) <> 0 And (InStr(1, OpenFile, Chr$(0)) > 0) Then
                Config.Root.Item("lastdir") = Mid$(OpenFile, 1, InStr(1, OpenFile, Chr$(0)) - 1)
            Else
                Config.Root.Item("lastdir") = Mid$(OpenFile, 1, InStrRev(OpenFile, "\") - 1)
            End If
                        
        End If
    
    End With
     
End Function


' Save a file
Public Function SaveFile(ByVal hWnd As Long, ByRef DialogTitle As String, ByRef Filter As String, ByRef InitFilename As String, ByVal flags As cdlFileOpenConstants, Optional ByRef FilterIndex As Long) As String
     
     Dim FileStruct As cdlOpenFileName
     Dim intNull As Integer
     
     
     With FileStruct
          
          ' Set all dialog parameters
          .lStructSize = Len(FileStruct)
          .hInstance = App.hInstance
          .Owner = hWnd
          .Title = DialogTitle
          .Filter = Replace$(Filter, "|", vbNullChar)
          .flags = flags
          .File = String$(257, Chr$(0)) & vbNullChar
          .MaxFile = Len(.File)
          .InitialDir = Config.Root.Item("lastdir")
          If InitFilename <> "" And Dir(InitFilename) <> "" Then .InitialDir = Left$(InitFilename, Len(InitFilename) - Len(Dir(InitFilename)))
          
          ' Show the dialog and return the result
          If GetSaveFileName(FileStruct) Then SaveFile = StripNullChar(.File)
          
          ' Return FilterIndex
          FilterIndex = .FilterIndex
          
     End With
     
End Function


